@@ -34,6 +34,7 @@
 #include <KIconEffect>
 #include <KIO/NetAccess>
 #include <KIO/PreviewJob>
+#include <KMessageWidget>
 #include <KNewFileMenu>
 #include <konqmimedata.h>
 #include <konq_operations.h>
@@ -44,11 +45,9 @@
 #include <KRun>
 
 #include "dolphin_generalsettings.h"
-#include "dolphinmainwindow.h"
 #include "filterbar/filterbar.h"
 #include "search/dolphinsearchbox.h"
 #include "statusbar/dolphinstatusbar.h"
-#include "views/dolphinplacesmodel.h"
 #include "views/draganddrophelper.h"
 #include "views/viewmodecontroller.h"
 #include "views/viewproperties.h"
@@ -58,11 +57,13 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
     m_topLayout(0),
     m_urlNavigator(0),
     m_searchBox(0),
+    m_messageWidget(0),
     m_view(0),
     m_filterBar(0),
     m_statusBar(0),
     m_statusBarTimer(0),
-    m_statusBarTimestamp()
+    m_statusBarTimestamp(),
+    m_autoGrabFocus(true)
 {
     hide();
 
@@ -70,7 +71,7 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
     m_topLayout->setSpacing(0);
     m_topLayout->setMargin(0);
 
-    m_urlNavigator = new KUrlNavigator(DolphinPlacesModel::instance(), url, this);
+    m_urlNavigator = new KUrlNavigator(new KFilePlacesModel(this), url, this);
     connect(m_urlNavigator, SIGNAL(urlsDropped(KUrl,QDropEvent*)),
             this, SLOT(dropUrls(KUrl,QDropEvent*)));
     connect(m_urlNavigator, SIGNAL(activated()),
@@ -88,27 +89,27 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
     m_searchBox = new DolphinSearchBox(this);
     m_searchBox->hide();
     connect(m_searchBox, SIGNAL(closeRequest()), this, SLOT(closeSearchBox()));
-    connect(m_searchBox, SIGNAL(search(QString)), this, SLOT(startSearching(QString)));
+    connect(m_searchBox, SIGNAL(searchRequest()), this, SLOT(startSearching()));
     connect(m_searchBox, SIGNAL(returnPressed(QString)), this, SLOT(requestFocus()));
 
+    m_messageWidget = new KMessageWidget(this);
+    m_messageWidget->setCloseButtonVisible(true);
+    m_messageWidget->hide();
+
     m_view = new DolphinView(url, this);
-    connect(m_view, SIGNAL(urlChanged(KUrl)),             m_urlNavigator, SLOT(setUrl(KUrl)));
-    connect(m_view, SIGNAL(writeStateChanged(bool)),      this, SIGNAL(writeStateChanged(bool)));
-    connect(m_view, SIGNAL(requestItemInfo(KFileItem)),   this, SLOT(showItemInfo(KFileItem)));
-    connect(m_view, SIGNAL(errorMessage(QString)),        this, SLOT(showErrorMessage(QString)));
-    connect(m_view, SIGNAL(infoMessage(QString)),         this, SLOT(showInfoMessage(QString)));
-    connect(m_view, SIGNAL(itemActivated(KFileItem)),     this, SLOT(slotItemActivated(KFileItem)));
-    connect(m_view, SIGNAL(redirection(KUrl,KUrl)),       this, SLOT(redirect(KUrl,KUrl)));
-    connect(m_view, SIGNAL(startedPathLoading(KUrl)),     this, SLOT(slotStartedPathLoading()));
-    connect(m_view, SIGNAL(finishedPathLoading(KUrl)),    this, SLOT(slotFinishedPathLoading()));
-    connect(m_view, SIGNAL(itemCountChanged()),           this, SLOT(delayedStatusBarUpdate()));
-    connect(m_view, SIGNAL(pathLoadingProgress(int)),     this, SLOT(updateProgress(int)));
-    connect(m_view, SIGNAL(infoMessage(QString)),         this, SLOT(showInfoMessage(QString)));
-    connect(m_view, SIGNAL(errorMessage(QString)),        this, SLOT(showErrorMessage(QString)));
-    connect(m_view, SIGNAL(urlIsFileError(KUrl)),         this, SLOT(openFile(KUrl)));
+    connect(m_view, SIGNAL(urlChanged(KUrl)),                   m_urlNavigator, SLOT(setUrl(KUrl)));
+    connect(m_view, SIGNAL(writeStateChanged(bool)),            this, SIGNAL(writeStateChanged(bool)));
+    connect(m_view, SIGNAL(requestItemInfo(KFileItem)),         this, SLOT(showItemInfo(KFileItem)));
+    connect(m_view, SIGNAL(itemActivated(KFileItem)),           this, SLOT(slotItemActivated(KFileItem)));
+    connect(m_view, SIGNAL(redirection(KUrl,KUrl)),             this, SLOT(redirect(KUrl,KUrl)));
+    connect(m_view, SIGNAL(directoryLoadingStarted()),          this, SLOT(slotDirectoryLoadingStarted()));
+    connect(m_view, SIGNAL(directoryLoadingCompleted()),        this, SLOT(slotDirectoryLoadingCompleted()));
+    connect(m_view, SIGNAL(itemCountChanged()),                 this, SLOT(delayedStatusBarUpdate()));
+    connect(m_view, SIGNAL(directoryLoadingProgress(int)),      this, SLOT(updateDirectoryLoadingProgress(int)));
+    connect(m_view, SIGNAL(directorySortingProgress(int)),      this, SLOT(updateDirectorySortingProgress(int)));
     connect(m_view, SIGNAL(selectionChanged(KFileItemList)),    this, SLOT(delayedStatusBarUpdate()));
-    connect(m_view, SIGNAL(operationCompletedMessage(QString)), this, SLOT(showOperationCompletedMessage(QString)));
     connect(m_view, SIGNAL(urlAboutToBeChanged(KUrl)),          this, SLOT(slotViewUrlAboutToBeChanged(KUrl)));
+    connect(m_view, SIGNAL(errorMessage(QString)),              this, SLOT(showErrorMessage(QString)));
 
     connect(m_urlNavigator, SIGNAL(urlAboutToBeChanged(KUrl)),
             this, SLOT(slotUrlNavigatorLocationAboutToBeChanged(KUrl)));
@@ -117,21 +118,27 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
     connect(m_urlNavigator, SIGNAL(historyChanged()),
             this, SLOT(slotHistoryChanged()));
 
-    // initialize status bar
-    m_statusBar = new DolphinStatusBar(this, m_view);
-    connect(m_statusBar, SIGNAL(stopPressed()), this, SLOT(stopLoading()));
+    // Initialize status bar
+    m_statusBar = new DolphinStatusBar(this);
+    m_statusBar->setUrl(m_view->url());
+    m_statusBar->setZoomLevel(m_view->zoomLevel());
+    connect(m_view, SIGNAL(urlChanged(KUrl)),                   m_statusBar, SLOT(setUrl(KUrl)));
+    connect(m_view, SIGNAL(zoomLevelChanged(int,int)),          m_statusBar, SLOT(setZoomLevel(int)));
+    connect(m_view, SIGNAL(infoMessage(QString)),               m_statusBar, SLOT(setText(QString)));
+    connect(m_view, SIGNAL(operationCompletedMessage(QString)), m_statusBar, SLOT(setText(QString)));
+    connect(m_statusBar, SIGNAL(stopPressed()),                 this, SLOT(stopDirectoryLoading()));
+    connect(m_statusBar, SIGNAL(zoomLevelChanged(int)),         this, SLOT(slotStatusBarZoomLevelChanged(int)));
 
     m_statusBarTimer = new QTimer(this);
     m_statusBarTimer->setSingleShot(true);
     m_statusBarTimer->setInterval(300);
-    connect(m_statusBarTimer, SIGNAL(timeout()),
-            this, SLOT(updateStatusBar()));
+    connect(m_statusBarTimer, SIGNAL(timeout()), this, SLOT(updateStatusBar()));
 
     KIO::FileUndoManager* undoManager = KIO::FileUndoManager::self();
     connect(undoManager, SIGNAL(jobRecordingFinished(CommandType)),
             this, SLOT(delayedStatusBarUpdate()));
 
-    // initialize filter bar
+    // Initialize filter bar
     m_filterBar = new FilterBar(this);
     m_filterBar->setVisible(settings->filterBar());
     connect(m_filterBar, SIGNAL(filterChanged(QString)),
@@ -143,6 +150,7 @@ DolphinViewContainer::DolphinViewContainer(const KUrl& url, QWidget* parent) :
 
     m_topLayout->addWidget(m_urlNavigator);
     m_topLayout->addWidget(m_searchBox);
+    m_topLayout->addWidget(m_messageWidget);
     m_topLayout->addWidget(m_view);
     m_topLayout->addWidget(m_filterBar);
     m_topLayout->addWidget(m_statusBar);
@@ -171,6 +179,16 @@ bool DolphinViewContainer::isActive() const
     return m_view->isActive();
 }
 
+void DolphinViewContainer::setAutoGrabFocus(bool grab)
+{
+    m_autoGrabFocus = grab;
+}
+
+bool DolphinViewContainer::autoGrabFocus() const
+{
+    return m_autoGrabFocus;
+}
+
 const DolphinStatusBar* DolphinViewContainer::statusBar() const
 {
     return m_statusBar;
@@ -201,14 +219,28 @@ DolphinView* DolphinViewContainer::view()
     return m_view;
 }
 
-const DolphinSearchBox* DolphinViewContainer::searchBox() const
+void DolphinViewContainer::showMessage(const QString& msg, MessageType type)
 {
-    return m_searchBox;
-}
+    if (msg.isEmpty()) {
+        return;
+    }
 
-DolphinSearchBox* DolphinViewContainer::searchBox()
-{
-    return m_searchBox;
+    m_messageWidget->setText(msg);
+
+    switch (type) {
+    case Information: m_messageWidget->setMessageType(KMessageWidget::Information); break;
+    case Warning:     m_messageWidget->setMessageType(KMessageWidget::Warning); break;
+    case Error:       m_messageWidget->setMessageType(KMessageWidget::Error); break;
+    default:
+        Q_ASSERT(false);
+        break;
+    }
+
+    m_messageWidget->setWordWrap(false);
+    const int unwrappedWidth = m_messageWidget->sizeHint().width();
+    m_messageWidget->setWordWrap(unwrappedWidth > size().width());
+
+    m_messageWidget->animatedShow();
 }
 
 void DolphinViewContainer::readSettings()
@@ -264,6 +296,8 @@ void DolphinViewContainer::setSearchModeEnabled(bool enabled)
             m_searchBox->setSearchPath(url);
         }
     } else {
+        m_view->setViewPropertiesContext(QString());
+
         // Restore the URL for the URL navigator. If Dolphin has been
         // started with a search-URL, the home URL is used as fallback.
         const KUrl url = m_searchBox->searchPath();
@@ -275,8 +309,6 @@ void DolphinViewContainer::setSearchModeEnabled(bool enabled)
             }
         }
     }
-
-    emit searchModeChanged(enabled);
 }
 
 bool DolphinViewContainer::isSearchModeEnabled() const
@@ -322,17 +354,12 @@ void DolphinViewContainer::updateStatusBar()
 {
     m_statusBarTimestamp.start();
 
-    const QString newMessage = m_view->statusBarText();
-    m_statusBar->setDefaultText(newMessage);
-
-    // We don't want to override errors. Other messages are only protected by
-    // the Statusbar itself depending on timings (see DolphinStatusBar::setMessage).
-    if (m_statusBar->type() != DolphinStatusBar::Error) {
-        m_statusBar->setMessage(newMessage, DolphinStatusBar::Default);
-    }
+    const QString text = m_view->statusBarText();
+    m_statusBar->setDefaultText(text);
+    m_statusBar->resetToDefaultText();
 }
 
-void DolphinViewContainer::updateProgress(int percent)
+void DolphinViewContainer::updateDirectoryLoadingProgress(int percent)
 {
     if (m_statusBar->progressText().isEmpty()) {
         m_statusBar->setProgressText(i18nc("@info:progress", "Loading folder..."));
@@ -340,7 +367,15 @@ void DolphinViewContainer::updateProgress(int percent)
     m_statusBar->setProgress(percent);
 }
 
-void DolphinViewContainer::slotStartedPathLoading()
+void DolphinViewContainer::updateDirectorySortingProgress(int percent)
+{
+    if (m_statusBar->progressText().isEmpty()) {
+        m_statusBar->setProgressText(i18nc("@info:progress", "Sorting..."));
+    }
+    m_statusBar->setProgress(percent);
+}
+
+void DolphinViewContainer::slotDirectoryLoadingStarted()
 {
     if (isSearchUrl(url())) {
         // Search KIO-slaves usually don't provide any progress information. Give
@@ -352,21 +387,21 @@ void DolphinViewContainer::slotStartedPathLoading()
         // Trigger an undetermined progress indication. The progress
         // information in percent will be triggered by the percent() signal
         // of the directory lister later.
-        updateProgress(-1);
+        updateDirectoryLoadingProgress(-1);
     }
 }
 
-void DolphinViewContainer::slotFinishedPathLoading()
+void DolphinViewContainer::slotDirectoryLoadingCompleted()
 {
     if (!m_statusBar->progressText().isEmpty()) {
         m_statusBar->setProgressText(QString());
         m_statusBar->setProgress(100);
     }
 
-    if (isSearchUrl(url()) && m_view->items().isEmpty()) {
+    if (isSearchUrl(url()) && m_view->itemsCount() == 0) {
         // The dir lister has been completed on a Nepomuk-URI and no items have been found. Instead
         // of showing the default status bar information ("0 items") a more helpful information is given:
-        m_statusBar->setMessage(i18nc("@info:status", "No items found."), DolphinStatusBar::Information);
+        m_statusBar->setText(i18nc("@info:status", "No items found."));
     } else {
         updateStatusBar();
     }
@@ -413,43 +448,16 @@ void DolphinViewContainer::slotItemActivated(const KFileItem& item)
     item.run();
 }
 
-void DolphinViewContainer::openFile(const KUrl& url)
-{
-    const KFileItem item(KFileItem::Unknown, KFileItem::Unknown, url);
-    slotItemActivated(item);
-}
-
 void DolphinViewContainer::showItemInfo(const KFileItem& item)
 {
     if (item.isNull()) {
-        // Only clear the status bar if unimportant messages are shown.
-        // This prevents that information- or error-messages get hidden
-        // by moving the mouse above the viewport or when closing the
-        // context menu.
-        if (m_statusBar->type() == DolphinStatusBar::Default) {
-            m_statusBar->clear();
-        }
+        m_statusBar->resetToDefaultText();
     } else {
-        const QString message = item.isDir() ? item.text() : item.getStatusBarInfo();
-        m_statusBar->setMessage(message, DolphinStatusBar::Default);
+        const QString text = item.isDir() ? item.text() : item.getStatusBarInfo();
+        m_statusBar->setText(text);
     }
 }
 
-void DolphinViewContainer::showInfoMessage(const QString& msg)
-{
-    m_statusBar->setMessage(msg, DolphinStatusBar::Information);
-}
-
-void DolphinViewContainer::showErrorMessage(const QString& msg)
-{
-    m_statusBar->setMessage(msg, DolphinStatusBar::Error);
-}
-
-void DolphinViewContainer::showOperationCompletedMessage(const QString& msg)
-{
-    m_statusBar->setMessage(msg, DolphinStatusBar::OperationCompleted);
-}
-
 void DolphinViewContainer::closeFilterBar()
 {
     m_filterBar->hide();
@@ -499,7 +507,7 @@ void DolphinViewContainer::slotUrlNavigatorLocationChanged(const KUrl& url)
         setSearchModeEnabled(isSearchUrl(url));
         m_view->setUrl(url);
 
-        if (isActive() && !isSearchUrl(url)) {
+        if (m_autoGrabFocus && isActive() && !isSearchUrl(url)) {
             // When an URL has been entered, the view should get the focus.
             // The focus must be requested asynchronously, as changing the URL might create
             // a new view widget.
@@ -508,8 +516,10 @@ void DolphinViewContainer::slotUrlNavigatorLocationChanged(const KUrl& url)
     } else if (KProtocolManager::isSourceProtocol(url)) {
         QString app = "konqueror";
         if (url.protocol().startsWith(QLatin1String("http"))) {
-            showErrorMessage(i18nc("@info:status",
-                                   "Dolphin does not support web pages, the web browser has been launched"));
+            showMessage(i18nc("@info:status",
+                              "Dolphin does not support web pages, the web browser has been launched"),
+                        Information);
+
             const KConfigGroup config(KSharedConfig::openConfig("kdeglobals"), "General");
             const QString browser = config.readEntry("BrowserApplication");
             if (!browser.isEmpty()) {
@@ -520,21 +530,25 @@ void DolphinViewContainer::slotUrlNavigatorLocationChanged(const KUrl& url)
                 }
             }
         } else {
-            showErrorMessage(i18nc("@info:status",
-                                   "Protocol not supported by Dolphin, Konqueror has been launched"));
+            showMessage(i18nc("@info:status",
+                              "Protocol not supported by Dolphin, Konqueror has been launched"),
+                        Information);
         }
 
         const QString secureUrl = KShell::quoteArg(url.pathOrUrl());
         const QString command = app + ' ' + secureUrl;
         KRun::runCommand(command, app, app, this);
     } else {
-        showErrorMessage(i18nc("@info:status", "Invalid protocol"));
+        showMessage(i18nc("@info:status", "Invalid protocol"), Error);
     }
 }
 
 void DolphinViewContainer::dropUrls(const KUrl& destination, QDropEvent* event)
 {
-    DragAndDropHelper::dropUrls(KFileItem(), destination, event);
+    const QString error = DragAndDropHelper::dropUrls(KFileItem(), destination, event);
+    if (!error.isEmpty()) {
+        showMessage(error, Error);
+    }
 }
 
 void DolphinViewContainer::redirect(const KUrl& oldUrl, const KUrl& newUrl)
@@ -572,11 +586,11 @@ void DolphinViewContainer::slotHistoryChanged()
     }
 }
 
-void DolphinViewContainer::startSearching(const QString &text)
+void DolphinViewContainer::startSearching()
 {
-    Q_UNUSED(text);
     const KUrl url = m_searchBox->urlForSearching();
     if (url.isValid() && !url.isEmpty()) {
+        m_view->setViewPropertiesContext("search");
         m_urlNavigator->setLocationUrl(url);
     }
 }
@@ -586,12 +600,22 @@ void DolphinViewContainer::closeSearchBox()
     setSearchModeEnabled(false);
 }
 
-void DolphinViewContainer::stopLoading()
+void DolphinViewContainer::stopDirectoryLoading()
 {
     m_view->stopLoading();
     m_statusBar->setProgress(100);
 }
 
+void DolphinViewContainer::slotStatusBarZoomLevelChanged(int zoomLevel)
+{
+    m_view->setZoomLevel(zoomLevel);
+}
+
+void DolphinViewContainer::showErrorMessage(const QString& msg)
+{
+    showMessage(msg, Error);
+}
+
 bool DolphinViewContainer::isSearchUrl(const KUrl& url) const
 {
     const QString protocol = url.protocol();
