@@ -19,24 +19,28 @@
 
 #include "kfileitemmodel.h"
 
-#include <KDirLister>
 #include <KDirModel>
 #include <KGlobalSettings>
 #include <KLocale>
 #include <KStringHandler>
 #include <KDebug>
 
+#include "private/kfileitemmodelsortalgorithm.h"
+#include "private/kfileitemmodeldirlister.h"
+
+#include <QApplication>
 #include <QMimeData>
 #include <QTimer>
 
 // #define KFILEITEMMODEL_DEBUG
 
-KFileItemModel::KFileItemModel(KDirLister* dirLister, QObject* parent) :
-    KItemModelBase("name", parent),
-    m_dirLister(dirLister),
+KFileItemModel::KFileItemModel(QObject* parent) :
+    KItemModelBase("text", parent),
+    m_dirLister(0),
     m_naturalSorting(KGlobalSettings::naturalSorting()),
-    m_sortFoldersFirst(true),
+    m_sortDirsFirst(true),
     m_sortRole(NameRole),
+    m_sortingProgressPercent(-1),
     m_roles(),
     m_caseSensitivity(Qt::CaseInsensitive),
     m_itemData(),
@@ -49,25 +53,34 @@ KFileItemModel::KFileItemModel(KDirLister* dirLister, QObject* parent) :
     m_pendingItemsToInsert(),
     m_groups(),
     m_expandedParentsCountRoot(UninitializedExpandedParentsCountRoot),
-    m_expandedUrls(),
+    m_expandedDirs(),
     m_urlsToExpand()
 {
+    m_dirLister = new KFileItemModelDirLister(this);
+    m_dirLister->setAutoUpdate(true);
+    m_dirLister->setDelayedMimeTypes(true);
+    m_dirLister->setMainWindow(qApp->activeWindow());
+
+    connect(m_dirLister, SIGNAL(started(KUrl)), this, SIGNAL(directoryLoadingStarted()));
+    connect(m_dirLister, SIGNAL(canceled()), this, SLOT(slotCanceled()));
+    connect(m_dirLister, SIGNAL(completed(KUrl)), this, SLOT(slotCompleted()));
+    connect(m_dirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
+    connect(m_dirLister, SIGNAL(itemsDeleted(KFileItemList)), this, SLOT(slotItemsDeleted(KFileItemList)));
+    connect(m_dirLister, SIGNAL(refreshItems(QList<QPair<KFileItem,KFileItem> >)), this, SLOT(slotRefreshItems(QList<QPair<KFileItem,KFileItem> >)));
+    connect(m_dirLister, SIGNAL(clear()), this, SLOT(slotClear()));
+    connect(m_dirLister, SIGNAL(clear(KUrl)), this, SLOT(slotClear(KUrl)));
+    connect(m_dirLister, SIGNAL(infoMessage(QString)), this, SIGNAL(infoMessage(QString)));
+    connect(m_dirLister, SIGNAL(errorMessage(QString)), this, SIGNAL(errorMessage(QString)));
+    connect(m_dirLister, SIGNAL(redirection(KUrl,KUrl)), this, SIGNAL(directoryRedirection(KUrl,KUrl)));
+
     // Apply default roles that should be determined
     resetRoles();
     m_requestRole[NameRole] = true;
     m_requestRole[IsDirRole] = true;
-    m_roles.insert("name");
+    m_requestRole[IsLinkRole] = true;
+    m_roles.insert("text");
     m_roles.insert("isDir");
-
-    Q_ASSERT(dirLister);
-
-    connect(dirLister, SIGNAL(canceled()), this, SLOT(slotCanceled()));
-    connect(dirLister, SIGNAL(completed(KUrl)), this, SLOT(slotCompleted()));
-    connect(dirLister, SIGNAL(newItems(KFileItemList)), this, SLOT(slotNewItems(KFileItemList)));
-    connect(dirLister, SIGNAL(itemsDeleted(KFileItemList)), this, SLOT(slotItemsDeleted(KFileItemList)));
-    connect(dirLister, SIGNAL(refreshItems(QList<QPair<KFileItem,KFileItem> >)), this, SLOT(slotRefreshItems(QList<QPair<KFileItem,KFileItem> >)));
-    connect(dirLister, SIGNAL(clear()), this, SLOT(slotClear()));
-    connect(dirLister, SIGNAL(clear(KUrl)), this, SLOT(slotClear(KUrl)));
+    m_roles.insert("isLink");
 
     // For slow KIO-slaves like used for searching it makes sense to show results periodically even
     // before the completed() or canceled() signal has been emitted.
@@ -94,6 +107,26 @@ KFileItemModel::~KFileItemModel()
     m_itemData.clear();
 }
 
+void KFileItemModel::loadDirectory(const KUrl& url)
+{
+    m_dirLister->openUrl(url);
+}
+
+void KFileItemModel::refreshDirectory(const KUrl& url)
+{
+    m_dirLister->openUrl(url, KDirLister::Reload);
+}
+
+KUrl KFileItemModel::directory() const
+{
+    return m_dirLister->url();
+}
+
+void KFileItemModel::cancelDirectoryLoading()
+{
+    m_dirLister->stop();
+}
+
 int KFileItemModel::count() const
 {
     return m_itemData.count();
@@ -143,49 +176,41 @@ bool KFileItemModel::setData(int index, const QHash<QByteArray, QVariant>& value
     return true;
 }
 
-void KFileItemModel::setSortFoldersFirst(bool foldersFirst)
+void KFileItemModel::setSortDirectoriesFirst(bool dirsFirst)
 {
-    if (foldersFirst != m_sortFoldersFirst) {
-        m_sortFoldersFirst = foldersFirst;
+    if (dirsFirst != m_sortDirsFirst) {
+        m_sortDirsFirst = dirsFirst;
         resortAllItems();
     }
 }
 
-bool KFileItemModel::sortFoldersFirst() const
+bool KFileItemModel::sortDirectoriesFirst() const
 {
-    return m_sortFoldersFirst;
+    return m_sortDirsFirst;
 }
 
 void KFileItemModel::setShowHiddenFiles(bool show)
 {
-    KDirLister* dirLister = m_dirLister.data();
-    if (dirLister) {
-        dirLister->setShowingDotFiles(show);
-        dirLister->emitChanges();
-        if (show) {
-            slotCompleted();
-        }
+    m_dirLister->setShowingDotFiles(show);
+    m_dirLister->emitChanges();
+    if (show) {
+        slotCompleted();
     }
 }
 
 bool KFileItemModel::showHiddenFiles() const
 {
-    const KDirLister* dirLister = m_dirLister.data();
-    return dirLister ? dirLister->showingDotFiles() : false;
+    return m_dirLister->showingDotFiles();
 }
 
-void KFileItemModel::setShowFoldersOnly(bool enabled)
+void KFileItemModel::setShowDirectoriesOnly(bool enabled)
 {
-    KDirLister* dirLister = m_dirLister.data();
-    if (dirLister) {
-        dirLister->setDirOnlyMode(enabled);
-    }
+    m_dirLister->setDirOnlyMode(enabled);
 }
 
-bool KFileItemModel::showFoldersOnly() const
+bool KFileItemModel::showDirectoriesOnly() const
 {
-    KDirLister* dirLister = m_dirLister.data();
-    return dirLister ? dirLister->dirOnlyMode() : false;
+    return m_dirLister->dirOnlyMode();
 }
 
 QMimeData* KFileItemModel::createMimeData(const QSet<int>& indexes) const
@@ -230,12 +255,12 @@ int KFileItemModel::indexForKeyboardSearch(const QString& text, int startFromInd
 {
     startFromIndex = qMax(0, startFromIndex);
     for (int i = startFromIndex; i < count(); ++i) {
-        if (data(i)["name"].toString().startsWith(text, Qt::CaseInsensitive)) {
+        if (data(i)["text"].toString().startsWith(text, Qt::CaseInsensitive)) {
             return i;
         }
     }
     for (int i = 0; i < startFromIndex; ++i) {
-        if (data(i)["name"].toString().startsWith(text, Qt::CaseInsensitive)) {
+        if (data(i)["text"].toString().startsWith(text, Qt::CaseInsensitive)) {
             return i;
         }
     }
@@ -245,34 +270,21 @@ int KFileItemModel::indexForKeyboardSearch(const QString& text, int startFromInd
 bool KFileItemModel::supportsDropping(int index) const
 {
     const KFileItem item = fileItem(index);
-    return item.isNull() ? false : item.isDir() || item.isDesktopFile();
+    return !item.isNull() && (item.isDir() || item.isDesktopFile());
 }
 
 QString KFileItemModel::roleDescription(const QByteArray& role) const
 {
-    QString descr;
-
-    switch (roleIndex(role)) {
-    case NameRole:        descr = i18nc("@item:intable", "Name"); break;
-    case SizeRole:        descr = i18nc("@item:intable", "Size"); break;
-    case DateRole:        descr = i18nc("@item:intable", "Date"); break;
-    case PermissionsRole: descr = i18nc("@item:intable", "Permissions"); break;
-    case OwnerRole:       descr = i18nc("@item:intable", "Owner"); break;
-    case GroupRole:       descr = i18nc("@item:intable", "Group"); break;
-    case TypeRole:        descr = i18nc("@item:intable", "Type"); break;
-    case DestinationRole: descr = i18nc("@item:intable", "Destination"); break;
-    case PathRole:        descr = i18nc("@item:intable", "Path"); break;
-    case CommentRole:     descr = i18nc("@item:intable", "Comment"); break;
-    case TagsRole:        descr = i18nc("@item:intable", "Tags"); break;
-    case RatingRole:      descr = i18nc("@item:intable", "Rating"); break;
-    case NoRole:          break;
-    case IsDirRole:       break;
-    case IsExpandedRole:  break;
-    case ExpandedParentsCountRole: break;
-    default:                 Q_ASSERT(false); break;
+    static QHash<QByteArray, QString> description;
+    if (description.isEmpty()) {
+        int count = 0;
+        const RoleInfoMap* map = rolesInfoMap(count);
+        for (int i = 0; i < count; ++i) {
+            description.insert(map[i].role, map[i].roleTranslation);
+        }
     }
 
-    return descr;
+    return description.value(role);
 }
 
 QList<QPair<int, QVariant> > KFileItemModel::groups() const
@@ -282,24 +294,13 @@ QList<QPair<int, QVariant> > KFileItemModel::groups() const
         QElapsedTimer timer;
         timer.start();
 #endif
-        switch (roleIndex(sortRole())) {
+        switch (typeForRole(sortRole())) {
         case NameRole:        m_groups = nameRoleGroups(); break;
         case SizeRole:        m_groups = sizeRoleGroups(); break;
         case DateRole:        m_groups = dateRoleGroups(); break;
         case PermissionsRole: m_groups = permissionRoleGroups(); break;
-        case OwnerRole:       m_groups = genericStringRoleGroups("owner"); break;
-        case GroupRole:       m_groups = genericStringRoleGroups("group"); break;
-        case TypeRole:        m_groups = genericStringRoleGroups("type"); break;
-        case DestinationRole: m_groups = genericStringRoleGroups("destination"); break;
-        case PathRole:        m_groups = genericStringRoleGroups("path"); break;
-        case CommentRole:     m_groups = genericStringRoleGroups("comment"); break;
-        case TagsRole:        m_groups = genericStringRoleGroups("tags"); break;
         case RatingRole:      m_groups = ratingRoleGroups(); break;
-        case NoRole:          break;
-        case IsDirRole:       break;
-        case IsExpandedRole:  break;
-        case ExpandedParentsCountRole: break;
-        default:                 Q_ASSERT(false); break;
+        default:              m_groups = genericStringRoleGroups(sortRole()); break;
         }
 
 #ifdef KFILEITEMMODEL_DEBUG
@@ -346,11 +347,7 @@ int KFileItemModel::index(const KUrl& url) const
 
 KFileItem KFileItemModel::rootItem() const
 {
-    const KDirLister* dirLister = m_dirLister.data();
-    if (dirLister) {
-        return dirLister->rootItem();
-    }
-    return KFileItem();
+    return m_dirLister->rootItem();
 }
 
 void KFileItemModel::clear()
@@ -381,7 +378,7 @@ void KFileItemModel::setRoles(const QSet<QByteArray>& roles)
     QSetIterator<QByteArray> it(roles);
     while (it.hasNext()) {
         const QByteArray& role = it.next();
-        m_requestRole[roleIndex(role)] = true;
+        m_requestRole[typeForRole(role)] = true;
     }
 
     if (count() > 0) {
@@ -413,21 +410,14 @@ bool KFileItemModel::setExpanded(int index, bool expanded)
         return false;
     }
 
-    KDirLister* dirLister = m_dirLister.data();
     const KUrl url = m_itemData.at(index)->item.url();
     if (expanded) {
-        m_expandedUrls.insert(url);
-
-        if (dirLister) {
-            dirLister->openUrl(url, KDirLister::Keep);
-            return true;
-        }
+        m_expandedDirs.insert(url);
+        m_dirLister->openUrl(url, KDirLister::Keep);
     } else {
-        m_expandedUrls.remove(url);
+        m_expandedDirs.remove(url);
+        m_dirLister->stop(url);
 
-        if (dirLister) {
-            dirLister->stop(url);
-        }
 
         KFileItemList itemsToRemove;
         const int expandedParentsCount = data(index)["expandedParentsCount"].toInt();
@@ -437,10 +427,9 @@ bool KFileItemModel::setExpanded(int index, bool expanded)
             ++index;
         }
         removeItems(itemsToRemove);
-        return true;
     }
 
-    return false;
+    return true;
 }
 
 bool KFileItemModel::isExpanded(int index) const
@@ -470,30 +459,25 @@ int KFileItemModel::expandedParentsCount(int index) const
     return 0;
 }
 
-QSet<KUrl> KFileItemModel::expandedUrls() const
+QSet<KUrl> KFileItemModel::expandedDirectories() const
 {
-    return m_expandedUrls;
+    return m_expandedDirs;
 }
 
-void KFileItemModel::restoreExpandedUrls(const QSet<KUrl>& urls)
+void KFileItemModel::restoreExpandedDirectories(const QSet<KUrl>& urls)
 {
     m_urlsToExpand = urls;
 }
 
-void KFileItemModel::expandParentItems(const KUrl& url)
+void KFileItemModel::expandParentDirectories(const KUrl& url)
 {
-    const KDirLister* dirLister = m_dirLister.data();
-    if (!dirLister) {
-        return;
-    }
-
-    const int pos = dirLister->url().path().length();
+    const int pos = m_dirLister->url().path().length();
 
     // Assure that each sub-path of the URL that should be
     // expanded is added to m_urlsToExpand. KDirLister
     // does not care whether the parent-URL has already been
     // expanded.
-    KUrl urlToExpand = dirLister->url();
+    KUrl urlToExpand = m_dirLister->url();
     const QStringList subDirs = url.path().mid(pos).split(QDir::separator());
     for (int i = 0; i < subDirs.count() - 1; ++i) {
         urlToExpand.addPath(subDirs.at(i));
@@ -559,6 +543,28 @@ QString KFileItemModel::nameFilter() const
     return m_filter.pattern();
 }
 
+QList<KFileItemModel::RoleInfo> KFileItemModel::rolesInformation()
+{
+    static QList<RoleInfo> rolesInfo;
+    if (rolesInfo.isEmpty()) {
+        int count = 0;
+        const RoleInfoMap* map = rolesInfoMap(count);
+        for (int i = 0; i < count; ++i) {
+            if (map[i].roleType != NoRole) {
+                RoleInfo info;
+                info.role = map[i].role;
+                info.translation = map[i].roleTranslation;
+                info.group = map[i].groupTranslation;
+                info.requiresNepomuk = map[i].requiresNepomuk;
+                info.requiresIndexer = map[i].requiresIndexer;
+                rolesInfo.append(info);
+            }
+        }
+    }
+
+    return rolesInfo;
+}
+
 void KFileItemModel::onGroupedSortingChanged(bool current)
 {
     Q_UNUSED(current);
@@ -568,7 +574,7 @@ void KFileItemModel::onGroupedSortingChanged(bool current)
 void KFileItemModel::onSortRoleChanged(const QByteArray& current, const QByteArray& previous)
 {
     Q_UNUSED(previous);
-    m_sortRole = roleIndex(current);
+    m_sortRole = typeForRole(current);
 
 #ifdef KFILEITEMMODEL_DEBUG
     if (!m_requestRole[m_sortRole]) {
@@ -615,7 +621,7 @@ void KFileItemModel::resortAllItems()
     m_items.clear();
 
     // Resort the items
-    sort(m_itemData.begin(), m_itemData.end());
+    KFileItemModelSortAlgorithm::sort(this, m_itemData.begin(), m_itemData.end());
     for (int i = 0; i < itemCount; ++i) {
         m_items.insert(m_itemData.at(i)->item.url(), i);
     }
@@ -649,7 +655,7 @@ void KFileItemModel::slotCompleted()
         // Note that the parent folder must be expanded before any of its subfolders become visible.
         // Therefore, some URLs in m_restoredExpandedUrls might not be visible yet
         // -> we expand the first visible URL we find in m_restoredExpandedUrls.
-        foreach(const KUrl& url, m_urlsToExpand) {
+        foreach (const KUrl& url, m_urlsToExpand) {
             const int index = m_items.value(url, -1);
             if (index >= 0) {
                 m_urlsToExpand.remove(url);
@@ -666,7 +672,7 @@ void KFileItemModel::slotCompleted()
         m_urlsToExpand.clear();
     }
 
-    emit loadingCompleted();
+    emit directoryLoadingCompleted();
 }
 
 void KFileItemModel::slotCanceled()
@@ -851,7 +857,7 @@ void KFileItemModel::slotClear()
         emit itemsRemoved(KItemRangeList() << KItemRange(0, removedCount));
     }
 
-    m_expandedUrls.clear();
+    m_expandedDirs.clear();
 }
 
 void KFileItemModel::slotClear(const KUrl& url)
@@ -879,6 +885,13 @@ void KFileItemModel::insertItems(const KFileItemList& items)
         return;
     }
 
+    if (m_sortRole == TypeRole) {
+        // Try to resolve the MIME-types synchronously to prevent a reordering of
+        // the items when sorting by type (per default MIME-types are resolved
+        // asynchronously by KFileItemModelRolesUpdater).
+        determineMimeTypes(items, 200);
+    }
+
 #ifdef KFILEITEMMODEL_DEBUG
     QElapsedTimer timer;
     timer.start();
@@ -889,7 +902,7 @@ void KFileItemModel::insertItems(const KFileItemList& items)
     m_groups.clear();
 
     QList<ItemData*> sortedItems = createItemDataList(items);
-    sort(sortedItems.begin(), sortedItems.end());
+    KFileItemModelSortAlgorithm::sort(this, sortedItems.begin(), sortedItems.end());
 
 #ifdef KFILEITEMMODEL_DEBUG
     kDebug() << "[TIME] Sorting:" << timer.elapsed();
@@ -968,7 +981,7 @@ void KFileItemModel::removeItems(const KFileItemList& items)
             sortedItems.append(m_itemData.at(index));
         }
     }
-    sort(sortedItems.begin(), sortedItems.end());
+    KFileItemModelSortAlgorithm::sort(this, sortedItems.begin(), sortedItems.end());
 
     QList<int> indexesToRemove;
     indexesToRemove.reserve(items.count());
@@ -1080,7 +1093,7 @@ void KFileItemModel::removeExpandedItems()
     removeItems(expandedItems);
 
     m_expandedParentsCountRoot = UninitializedExpandedParentsCountRoot;
-    m_expandedUrls.clear();
+    m_expandedDirs.clear();
 }
 
 void KFileItemModel::resetRoles()
@@ -1090,52 +1103,56 @@ void KFileItemModel::resetRoles()
     }
 }
 
-KFileItemModel::Role KFileItemModel::roleIndex(const QByteArray& role) const
+KFileItemModel::RoleType KFileItemModel::typeForRole(const QByteArray& role) const
 {
-    static QHash<QByteArray, Role> rolesHash;
-    if (rolesHash.isEmpty()) {
-        rolesHash.insert("name", NameRole);
-        rolesHash.insert("size", SizeRole);
-        rolesHash.insert("date", DateRole);
-        rolesHash.insert("permissions", PermissionsRole);
-        rolesHash.insert("owner", OwnerRole);
-        rolesHash.insert("group", GroupRole);
-        rolesHash.insert("type", TypeRole);
-        rolesHash.insert("destination", DestinationRole);
-        rolesHash.insert("path", PathRole);
-        rolesHash.insert("comment", CommentRole);
-        rolesHash.insert("tags", TagsRole);
-        rolesHash.insert("rating", RatingRole);
-        rolesHash.insert("isDir", IsDirRole);
-        rolesHash.insert("isExpanded", IsExpandedRole);
-        rolesHash.insert("isExpandable", IsExpandableRole);
-        rolesHash.insert("expandedParentsCount", ExpandedParentsCountRole);
+    static QHash<QByteArray, RoleType> roles;
+    if (roles.isEmpty()) {
+        // Insert user visible roles that can be accessed with
+        // KFileItemModel::roleInformation()
+        int count = 0;
+        const RoleInfoMap* map = rolesInfoMap(count);
+        for (int i = 0; i < count; ++i) {
+            roles.insert(map[i].role, map[i].roleType);
+        }
+
+        // Insert internal roles (take care to synchronize the implementation
+        // with KFileItemModel::roleForType() in case if a change is done).
+        roles.insert("isDir", IsDirRole);
+        roles.insert("isLink", IsLinkRole);
+        roles.insert("isExpanded", IsExpandedRole);
+        roles.insert("isExpandable", IsExpandableRole);
+        roles.insert("expandedParentsCount", ExpandedParentsCountRole);
+
+        Q_ASSERT(roles.count() == RolesCount);
     }
-    return rolesHash.value(role, NoRole);
+
+    return roles.value(role, NoRole);
 }
 
-QByteArray KFileItemModel::roleByteArray(Role role) const
+QByteArray KFileItemModel::roleForType(RoleType roleType) const
 {
-    static const char* const roles[RolesCount] = {
-        0, // NoRole
-        "name",
-        "size",
-        "date",
-        "permissions",
-        "owner",
-        "group",
-        "type",
-        "destination",
-        "path",
-        "comment",
-        "tags",
-        "rating",
-        "isDir",
-        "isExpanded",
-        "isExpandable",
-        "expandedParentsCount"
+    static QHash<RoleType, QByteArray> roles;
+    if (roles.isEmpty()) {
+        // Insert user visible roles that can be accessed with
+        // KFileItemModel::roleInformation()
+        int count = 0;
+        const RoleInfoMap* map = rolesInfoMap(count);
+        for (int i = 0; i < count; ++i) {
+            roles.insert(map[i].roleType, map[i].role);
+        }
+
+        // Insert internal roles (take care to synchronize the implementation
+        // with KFileItemModel::typeForRole() in case if a change is done).
+        roles.insert(IsDirRole, "isDir");
+        roles.insert(IsLinkRole, "isLink");
+        roles.insert(IsExpandedRole, "isExpanded");
+        roles.insert(IsExpandableRole, "isExpandable");
+        roles.insert(ExpandedParentsCountRole, "expandedParentsCount");
+
+        Q_ASSERT(roles.count() == RolesCount);
     };
-    return roles[role];
+
+    return roles.value(roleType);
 }
 
 QHash<QByteArray, QVariant> KFileItemModel::retrieveData(const KFileItem& item) const
@@ -1151,8 +1168,13 @@ QHash<QByteArray, QVariant> KFileItemModel::retrieveData(const KFileItem& item)
         data.insert("isDir", isDir);
     }
 
+    if (m_requestRole[IsLinkRole]) {
+        const bool isLink = item.isLink();
+        data.insert("isLink", isLink);
+    }
+
     if (m_requestRole[NameRole]) {
-        data.insert("name", item.text());
+        data.insert("text", item.text());
     }
 
     if (m_requestRole[SizeRole]) {
@@ -1186,7 +1208,7 @@ QHash<QByteArray, QVariant> KFileItemModel::retrieveData(const KFileItem& item)
     if (m_requestRole[DestinationRole]) {
         QString destination = item.linkDest();
         if (destination.isEmpty()) {
-            destination = i18nc("@item:intable", "No destination");
+            destination = QLatin1String("-");
         }
         data.insert("destination", destination);
     }
@@ -1196,7 +1218,17 @@ QHash<QByteArray, QVariant> KFileItemModel::retrieveData(const KFileItem& item)
         if (item.url().protocol() == QLatin1String("trash")) {
             path = item.entry().stringValue(KIO::UDSEntry::UDS_EXTRA);
         } else {
+            // For performance reasons cache the home-path in a static QString
+            // (see QDir::homePath() for more details)
+            static QString homePath;
+            if (homePath.isEmpty()) {
+                homePath = QDir::homePath();
+            }
+
             path = item.localPath();
+            if (path.startsWith(homePath)) {
+                path.replace(0, homePath.length(), QLatin1Char('~'));
+            }
         }
 
         const int index = path.lastIndexOf(item.text());
@@ -1213,8 +1245,8 @@ QHash<QByteArray, QVariant> KFileItemModel::retrieveData(const KFileItem& item)
     }
 
     if (m_requestRole[ExpandedParentsCountRole]) {
-        if (m_expandedParentsCountRoot == UninitializedExpandedParentsCountRoot && m_dirLister.data()) {
-            const KUrl rootUrl = m_dirLister.data()->url();
+        if (m_expandedParentsCountRoot == UninitializedExpandedParentsCountRoot) {
+            const KUrl rootUrl = m_dirLister->url();
             const QString protocol = rootUrl.protocol();
             const bool forceExpandedParentsCountRoot = (protocol == QLatin1String("trash") ||
                                                         protocol == QLatin1String("nepomuk") ||
@@ -1260,7 +1292,7 @@ bool KFileItemModel::lessThan(const ItemData* a, const ItemData* b) const
         }
     }
 
-    if (m_sortFoldersFirst || m_sortRole == SizeRole) {
+    if (m_sortDirsFirst || m_sortRole == SizeRole) {
         const bool isDirA = a->item.isDir();
         const bool isDirB = b->item.isDir();
         if (isDirA && !isDirB) {
@@ -1335,22 +1367,22 @@ int KFileItemModel::sortRoleCompare(const ItemData* a, const ItemData* b) const
         break;
     }
 
-    case PermissionsRole:
-    case OwnerRole:
-    case GroupRole:
-    case TypeRole:
-    case DestinationRole:
-    case PathRole:
-    case CommentRole:
-    case TagsRole: {
-        const QByteArray role = roleByteArray(m_sortRole);
+    case ImageSizeRole: {
+        // Alway use a natural comparing to interpret the numbers of a string like
+        // "1600 x 1200" for having a correct sorting.
+        result = KStringHandler::naturalCompare(a->values.value("imageSize").toString(),
+                                                b->values.value("imageSize").toString(),
+                                                Qt::CaseSensitive);
+        break;
+    }
+
+    default: {
+        const QByteArray role = roleForType(m_sortRole);
         result = QString::compare(a->values.value(role).toString(),
                                   b->values.value(role).toString());
         break;
     }
 
-    default:
-        break;
     }
 
     if (result != 0) {
@@ -1377,128 +1409,6 @@ int KFileItemModel::sortRoleCompare(const ItemData* a, const ItemData* b) const
     return QString::compare(itemA.url().url(), itemB.url().url(), Qt::CaseSensitive);
 }
 
-void KFileItemModel::sort(QList<ItemData*>::iterator begin,
-                          QList<ItemData*>::iterator end)
-{
-    // The implementation is based on qStableSortHelper() from qalgorithms.h
-    // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-    // In opposite to qStableSort() it allows to use a member-function for the comparison of elements.
-
-    const int span = end - begin;
-    if (span < 2) {
-        return;
-    }
-
-    const QList<ItemData*>::iterator middle = begin + span / 2;
-    sort(begin, middle);
-    sort(middle, end);
-    merge(begin, middle, end);
-}
-
-void KFileItemModel::merge(QList<ItemData*>::iterator begin,
-                           QList<ItemData*>::iterator pivot,
-                           QList<ItemData*>::iterator end)
-{
-    // The implementation is based on qMerge() from qalgorithms.h
-    // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-
-    const int len1 = pivot - begin;
-    const int len2 = end - pivot;
-
-    if (len1 == 0 || len2 == 0) {
-        return;
-    }
-
-    if (len1 + len2 == 2) {
-        if (lessThan(*(begin + 1), *(begin))) {
-            qSwap(*begin, *(begin + 1));
-        }
-        return;
-    }
-
-    QList<ItemData*>::iterator firstCut;
-    QList<ItemData*>::iterator secondCut;
-    int len2Half;
-    if (len1 > len2) {
-        const int len1Half = len1 / 2;
-        firstCut = begin + len1Half;
-        secondCut = lowerBound(pivot, end, *firstCut);
-        len2Half = secondCut - pivot;
-    } else {
-        len2Half = len2 / 2;
-        secondCut = pivot + len2Half;
-        firstCut = upperBound(begin, pivot, *secondCut);
-    }
-
-    reverse(firstCut, pivot);
-    reverse(pivot, secondCut);
-    reverse(firstCut, secondCut);
-
-    const QList<ItemData*>::iterator newPivot = firstCut + len2Half;
-    merge(begin, firstCut, newPivot);
-    merge(newPivot, secondCut, end);
-}
-
-QList<KFileItemModel::ItemData*>::iterator KFileItemModel::lowerBound(QList<ItemData*>::iterator begin,
-                                                                      QList<ItemData*>::iterator end,
-                                                                      const ItemData* value)
-{
-    // The implementation is based on qLowerBound() from qalgorithms.h
-    // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-
-    QList<ItemData*>::iterator middle;
-    int n = int(end - begin);
-    int half;
-
-    while (n > 0) {
-        half = n >> 1;
-        middle = begin + half;
-        if (lessThan(*middle, value)) {
-            begin = middle + 1;
-            n -= half + 1;
-        } else {
-            n = half;
-        }
-    }
-    return begin;
-}
-
-QList<KFileItemModel::ItemData*>::iterator KFileItemModel::upperBound(QList<ItemData*>::iterator begin,
-                                                                      QList<ItemData*>::iterator end,
-                                                                      const ItemData* value)
-{
-    // The implementation is based on qUpperBound() from qalgorithms.h
-    // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-
-    QList<ItemData*>::iterator middle;
-    int n = end - begin;
-    int half;
-
-    while (n > 0) {
-        half = n >> 1;
-        middle = begin + half;
-        if (lessThan(value, *middle)) {
-            n = half;
-        } else {
-            begin = middle + 1;
-            n -= half + 1;
-        }
-    }
-    return begin;
-}
-
-void KFileItemModel::reverse(QList<ItemData*>::iterator begin,
-                             QList<ItemData*>::iterator end)
-{
-    // The implementation is based on qReverse() from qalgorithms.h
-    // Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
-
-    --end;
-    while (begin < end) {
-        qSwap(*begin++, *end--);
-    }
-}
-
 int KFileItemModel::stringCompare(const QString& a, const QString& b) const
 {
     // Taken from KDirSortFilterProxyModel (kdelibs/kfile/kdirsortfilterproxymodel.*)
@@ -1561,7 +1471,7 @@ int KFileItemModel::expandedParentsCountCompare(const ItemData* a, const ItemDat
     bool isDirB = true;
     const QString subPathB = subPath(b->item, pathB, index, &isDirB);
 
-    if (m_sortFoldersFirst || m_sortRole == SizeRole) {
+    if (m_sortDirsFirst || m_sortRole == SizeRole) {
         if (isDirA && !isDirB) {
             return (sortOrder() == Qt::AscendingOrder) ? -1 : +1;
         } else if (!isDirA && isDirB) {
@@ -1605,8 +1515,7 @@ QString KFileItemModel::subPath(const KFileItem& item,
 
 bool KFileItemModel::useMaximumUpdateInterval() const
 {
-    const KDirLister* dirLister = m_dirLister.data();
-    return dirLister && !dirLister->url().isLocalFile();
+    return !m_dirLister->url().isLocalFile();
 }
 
 QList<QPair<int, QVariant> > KFileItemModel::nameRoleGroups() const
@@ -1624,7 +1533,7 @@ QList<QPair<int, QVariant> > KFileItemModel::nameRoleGroups() const
             continue;
         }
 
-        const QString name = m_itemData.at(i)->values.value("name").toString();
+        const QString name = m_itemData.at(i)->values.value("text").toString();
 
         // Use the first character of the name as group indication
         QChar newFirstChar = name.at(0).toUpper();
@@ -1941,4 +1850,78 @@ KFileItemList KFileItemModel::childItems(const KFileItem& item) const
     return items;
 }
 
+void KFileItemModel::emitSortProgress(int resolvedCount)
+{
+    // Be tolerant against a resolvedCount with a wrong range.
+    // Although there should not be a case where KFileItemModelRolesUpdater
+    // (= caller) provides a wrong range, it is important to emit
+    // a useful progress information even if there is an unexpected
+    // implementation issue.
+
+    const int itemCount = count();
+    if (resolvedCount >= itemCount) {
+        m_sortingProgressPercent = -1;
+        if (m_resortAllItemsTimer->isActive()) {
+            m_resortAllItemsTimer->stop();
+            resortAllItems();
+        }
+
+        emit directorySortingProgress(100);
+    } else if (itemCount > 0) {
+        resolvedCount = qBound(0, resolvedCount, itemCount);
+
+        const int progress = resolvedCount * 100 / itemCount;
+        if (m_sortingProgressPercent != progress) {
+            m_sortingProgressPercent = progress;
+            emit directorySortingProgress(progress);
+        }
+    }
+}
+
+const KFileItemModel::RoleInfoMap* KFileItemModel::rolesInfoMap(int& count)
+{
+    static const RoleInfoMap rolesInfoMap[] = {
+    //  | role         | roleType       | role translation                                | group translation           | requires Nepomuk | requires indexer
+        { 0,             NoRole,          0, 0,                                             0, 0,                                     false, false },
+        { "text",        NameRole,        I18N_NOOP2_NOSTRIP("@label", "Name"),             0, 0,                                     false, false },
+        { "size",        SizeRole,        I18N_NOOP2_NOSTRIP("@label", "Size"),             0, 0,                                     false, false },
+        { "date",        DateRole,        I18N_NOOP2_NOSTRIP("@label", "Date"),             0, 0,                                     false, false },
+        { "type",        TypeRole,        I18N_NOOP2_NOSTRIP("@label", "Type"),             0, 0,                                     false, false },
+        { "rating",      RatingRole,      I18N_NOOP2_NOSTRIP("@label", "Rating"),           0, 0,                                     true,  false },
+        { "tags",        TagsRole,        I18N_NOOP2_NOSTRIP("@label", "Tags"),             0, 0,                                     true,  false },
+        { "comment",     CommentRole,     I18N_NOOP2_NOSTRIP("@label", "Comment"),          0, 0,                                     true,  false },
+        { "wordCount",   WordCountRole,   I18N_NOOP2_NOSTRIP("@label", "Word Count"),       I18N_NOOP2_NOSTRIP("@label", "Document"), true,  true  },
+        { "lineCount",   LineCountRole,   I18N_NOOP2_NOSTRIP("@label", "Line Count"),       I18N_NOOP2_NOSTRIP("@label", "Document"), true,  true  },
+        { "imageSize",   ImageSizeRole,   I18N_NOOP2_NOSTRIP("@label", "Image Size"),       I18N_NOOP2_NOSTRIP("@label", "Image"),    true,  true  },
+        { "orientation", OrientationRole, I18N_NOOP2_NOSTRIP("@label", "Orientation"),      I18N_NOOP2_NOSTRIP("@label", "Image"),    true,  true  },
+        { "artist",      ArtistRole,      I18N_NOOP2_NOSTRIP("@label", "Artist"),           I18N_NOOP2_NOSTRIP("@label", "Audio"),    true,  true  },
+        { "album",       AlbumRole,       I18N_NOOP2_NOSTRIP("@label", "Album"),            I18N_NOOP2_NOSTRIP("@label", "Audio"),    true,  true  },
+        { "duration",    DurationRole,    I18N_NOOP2_NOSTRIP("@label", "Duration"),         I18N_NOOP2_NOSTRIP("@label", "Audio"),    true,  true  },
+        { "track",       TrackRole,       I18N_NOOP2_NOSTRIP("@label", "Track"),            I18N_NOOP2_NOSTRIP("@label", "Audio"),    true,  true  },
+        { "path",        PathRole,        I18N_NOOP2_NOSTRIP("@label", "Path"),             I18N_NOOP2_NOSTRIP("@label", "Other"),    false, false },
+        { "destination", DestinationRole, I18N_NOOP2_NOSTRIP("@label", "Link Destination"), I18N_NOOP2_NOSTRIP("@label", "Other"),    false, false },
+        { "copiedFrom",  CopiedFromRole,  I18N_NOOP2_NOSTRIP("@label", "Copied From"),      I18N_NOOP2_NOSTRIP("@label", "Other"),    true,  false },
+        { "permissions", PermissionsRole, I18N_NOOP2_NOSTRIP("@label", "Permissions"),      I18N_NOOP2_NOSTRIP("@label", "Other"),    false, false },
+        { "owner",       OwnerRole,       I18N_NOOP2_NOSTRIP("@label", "Owner"),            I18N_NOOP2_NOSTRIP("@label", "Other"),    false, false },
+        { "group",       GroupRole,       I18N_NOOP2_NOSTRIP("@label", "User Group"),       I18N_NOOP2_NOSTRIP("@label", "Other"),    false, false },
+    };
+
+    count = sizeof(rolesInfoMap) / sizeof(RoleInfoMap);
+    return rolesInfoMap;
+}
+
+void KFileItemModel::determineMimeTypes(const KFileItemList& items, int timeout)
+{
+    QElapsedTimer timer;
+    timer.start();
+    foreach (KFileItem item, items) {
+        item.determineMimeType();
+        if (timer.elapsed() > timeout) {
+            // Don't block the user interface, let the remaining items
+            // be resolved asynchronously.
+            return;
+        }
+    }
+}
+
 #include "kfileitemmodel.moc"
