@@ -20,19 +20,28 @@
 #include "kfileitemmodelrolesupdater.h"
 
 #include "kfileitemmodel.h"
-#include "kpixmapmodifier_p.h"
 
 #include <KConfig>
 #include <KConfigGroup>
 #include <KDebug>
 #include <KFileItem>
 #include <KGlobal>
+#include <KIO/JobUiDelegate>
 #include <KIO/PreviewJob>
+
+#include "private/kpixmapmodifier.h"
+
+#include <QApplication>
 #include <QPainter>
 #include <QPixmap>
 #include <QElapsedTimer>
 #include <QTimer>
 
+#ifdef HAVE_NEPOMUK
+    #include "private/knepomukrolesprovider.h"
+    #include "private/nepomuk/resourcewatcher.h"
+#endif
+
 // Required includes for subItemsCount():
 #ifdef Q_WS_WIN
     #include <QDir>
@@ -64,6 +73,7 @@ KFileItemModelRolesUpdater::KFileItemModelRolesUpdater(KFileItemModel* model, QO
     m_previewShown(false),
     m_enlargeSmallPreviews(true),
     m_clearPreviews(false),
+    m_sortingProgress(-1),
     m_model(model),
     m_iconSize(),
     m_firstVisibleIndex(0),
@@ -75,6 +85,11 @@ KFileItemModelRolesUpdater::KFileItemModelRolesUpdater(KFileItemModel* model, QO
     m_previewJobs(),
     m_changedItemsTimer(0),
     m_changedItems()
+  #ifdef HAVE_NEPOMUK
+  , m_nepomukResourceWatcher(0),
+    m_nepomukUriItems()
+  #endif
+
 {
     Q_ASSERT(model);
 
@@ -90,6 +105,8 @@ KFileItemModelRolesUpdater::KFileItemModelRolesUpdater(KFileItemModel* model, QO
             this,    SLOT(slotItemsRemoved(KItemRangeList)));
     connect(m_model, SIGNAL(itemsChanged(KItemRangeList,QSet<QByteArray>)),
             this,    SLOT(slotItemsChanged(KItemRangeList,QSet<QByteArray>)));
+    connect(m_model, SIGNAL(sortRoleChanged(QByteArray,QByteArray)),
+            this,    SLOT(slotSortRoleChanged(QByteArray,QByteArray)));
 
     // Use a timer to prevent that each call of slotItemsChanged() results in a synchronous
     // resolving of the roles. Postpone the resolving until no update has been done for 1 second.
@@ -97,6 +114,13 @@ KFileItemModelRolesUpdater::KFileItemModelRolesUpdater(KFileItemModel* model, QO
     m_changedItemsTimer->setInterval(1000);
     m_changedItemsTimer->setSingleShot(true);
     connect(m_changedItemsTimer, SIGNAL(timeout()), this, SLOT(resolveChangedItems()));
+
+    m_resolvableRoles.insert("size");
+    m_resolvableRoles.insert("type");
+    m_resolvableRoles.insert("isExpandable");
+#ifdef HAVE_NEPOMUK
+    m_resolvableRoles += KNepomukRolesProvider::instance().roles();
+#endif
 }
 
 KFileItemModelRolesUpdater::~KFileItemModelRolesUpdater()
@@ -225,6 +249,43 @@ void KFileItemModelRolesUpdater::setRoles(const QSet<QByteArray>& roles)
     if (m_roles != roles) {
         m_roles = roles;
 
+#ifdef HAVE_NEPOMUK
+        // Check whether there is at least one role that must be resolved
+        // with the help of Nepomuk. If this is the case, a (quite expensive)
+        // resolving will be done in KFileItemModelRolesUpdater::rolesData() and
+        // the role gets watched for changes.
+        const KNepomukRolesProvider& rolesProvider = KNepomukRolesProvider::instance();
+        bool hasNepomukRole = false;
+        QSetIterator<QByteArray> it(roles);
+        while (it.hasNext()) {
+            const QByteArray& role = it.next();
+            if (rolesProvider.roles().contains(role)) {
+                hasNepomukRole = true;
+                break;
+            }
+        }
+
+        if (hasNepomukRole && !m_nepomukResourceWatcher) {
+            Q_ASSERT(m_nepomukUriItems.isEmpty());
+
+            m_nepomukResourceWatcher = new Nepomuk::ResourceWatcher(this);
+            connect(m_nepomukResourceWatcher, SIGNAL(propertyChanged(Nepomuk::Resource,Nepomuk::Types::Property,QVariantList,QVariantList)),
+                    this, SLOT(applyChangedNepomukRoles(Nepomuk::Resource)));
+            connect(m_nepomukResourceWatcher, SIGNAL(propertyRemoved(Nepomuk::Resource,Nepomuk::Types::Property,QVariant)),
+                    this, SLOT(applyChangedNepomukRoles(Nepomuk::Resource)));
+            connect(m_nepomukResourceWatcher, SIGNAL(propertyAdded(Nepomuk::Resource,Nepomuk::Types::Property,QVariant)),
+                    this, SLOT(applyChangedNepomukRoles(Nepomuk::Resource)));
+            connect(m_nepomukResourceWatcher, SIGNAL(resourceCreated(Nepomuk::Resource,QList<QUrl>)),
+                    this, SLOT(applyChangedNepomukRoles(Nepomuk::Resource)));
+        } else if (!hasNepomukRole && m_nepomukResourceWatcher) {
+            delete m_nepomukResourceWatcher;
+            m_nepomukResourceWatcher = 0;
+            m_nepomukUriItems.clear();
+        }
+#endif
+
+        updateSortProgress();
+
         if (m_paused) {
             m_rolesChangedDuringPausing = true;
         } else {
@@ -256,6 +317,35 @@ void KFileItemModelRolesUpdater::slotItemsInserted(const KItemRangeList& itemRan
 void KFileItemModelRolesUpdater::slotItemsRemoved(const KItemRangeList& itemRanges)
 {
     Q_UNUSED(itemRanges);
+
+#ifdef HAVE_NEPOMUK
+    if (m_nepomukResourceWatcher) {
+        // Don't let the ResourceWatcher watch for removed items
+        if (m_model->count() == 0) {
+            m_nepomukResourceWatcher->setResources(QList<Nepomuk::Resource>());
+            m_nepomukResourceWatcher->stop();
+            m_nepomukUriItems.clear();
+        } else {
+            QList<Nepomuk::Resource> newResources;
+            const QList<Nepomuk::Resource> oldResources = m_nepomukResourceWatcher->resources();
+            foreach (const Nepomuk::Resource& resource, oldResources) {
+                const QUrl uri = resource.resourceUri();
+                const KUrl itemUrl = m_nepomukUriItems.value(uri);
+                if (m_model->index(itemUrl) >= 0) {
+                    newResources.append(resource);
+                } else {
+                    m_nepomukUriItems.remove(uri);
+                }
+            }
+            m_nepomukResourceWatcher->setResources(newResources);
+            if (newResources.isEmpty()) {
+                Q_ASSERT(m_nepomukUriItems.isEmpty());
+                m_nepomukResourceWatcher->stop();
+            }
+        }
+    }
+#endif
+
     m_firstVisibleIndex = 0;
     m_lastVisibleIndex = -1;
     if (!hasPendingRoles()) {
@@ -309,6 +399,14 @@ void KFileItemModelRolesUpdater::slotItemsChanged(const KItemRangeList& itemRang
     m_changedItemsTimer->start();
 }
 
+void KFileItemModelRolesUpdater::slotSortRoleChanged(const QByteArray& current,
+                                                     const QByteArray& previous)
+{
+    Q_UNUSED(current);
+    Q_UNUSED(previous);
+    updateSortProgress();
+}
+
 void KFileItemModelRolesUpdater::slotGotPreview(const KFileItem& item, const QPixmap& pixmap)
 {
     m_pendingVisibleItems.remove(item);
@@ -365,6 +463,8 @@ void KFileItemModelRolesUpdater::slotGotPreview(const KFileItem& item, const QPi
     m_model->setData(index, data);
     connect(m_model, SIGNAL(itemsChanged(KItemRangeList,QSet<QByteArray>)),
             this,    SLOT(slotItemsChanged(KItemRangeList,QSet<QByteArray>)));
+
+    applySortProgressToModel();
 }
 
 void KFileItemModelRolesUpdater::slotPreviewFailed(const KFileItem& item)
@@ -376,6 +476,8 @@ void KFileItemModelRolesUpdater::slotPreviewFailed(const KFileItem& item)
     m_clearPreviews = true;
     applyResolvedRoles(item, ResolveAll);
     m_clearPreviews = clearPreviews;
+
+    applySortProgressToModel();
 }
 
 void KFileItemModelRolesUpdater::slotPreviewJobFinished(KJob* job)
@@ -425,6 +527,8 @@ void KFileItemModelRolesUpdater::resolveNextPendingRoles()
         m_clearPreviews = false;
     }
 
+    applySortProgressToModel();
+
 #ifdef KFILEITEMMODELROLESUPDATER_DEBUG
     static int callCount = 0;
     ++callCount;
@@ -456,6 +560,31 @@ void KFileItemModelRolesUpdater::resolveChangedItems()
     startUpdating(itemRanges);
 }
 
+void KFileItemModelRolesUpdater::applyChangedNepomukRoles(const Nepomuk::Resource& resource)
+{
+#ifdef HAVE_NEPOMUK
+    const KUrl itemUrl = m_nepomukUriItems.value(resource.resourceUri());
+    const KFileItem item = m_model->fileItem(itemUrl);
+    QHash<QByteArray, QVariant> data = rolesData(item);
+
+    const KNepomukRolesProvider& rolesProvider = KNepomukRolesProvider::instance();
+    QHashIterator<QByteArray, QVariant> it(rolesProvider.roleValues(resource, m_roles));
+    while (it.hasNext()) {
+        it.next();
+        data.insert(it.key(), it.value());
+    }
+
+    disconnect(m_model, SIGNAL(itemsChanged(KItemRangeList,QSet<QByteArray>)),
+               this,    SLOT(slotItemsChanged(KItemRangeList,QSet<QByteArray>)));
+    const int index = m_model->index(item);
+    m_model->setData(index, data);
+    connect(m_model, SIGNAL(itemsChanged(KItemRangeList,QSet<QByteArray>)),
+            this,    SLOT(slotItemsChanged(KItemRangeList,QSet<QByteArray>)));
+#else
+    Q_UNUSED(resource);
+#endif
+}
+
 void KFileItemModelRolesUpdater::startUpdating(const KItemRangeList& itemRanges)
 {
     // If no valid index range is given assume that all items are visible.
@@ -514,7 +643,7 @@ void KFileItemModelRolesUpdater::startPreviewJob(const KFileItemList& items)
     // worst case) might block the application for several seconds. To prevent such
     // a blocking the MIME-type of the items will determined until the MaxBlockTimeout
     // has been reached and only those items will get passed. As soon as the MIME-type
-    // has been resolved once KIO::filePreview() can already access the resolved
+    // has been resolved once KIO::PreviewJob() can already access the resolved
     // MIME-type in a fast way.
     QElapsedTimer timer;
     timer.start();
@@ -531,7 +660,11 @@ void KFileItemModelRolesUpdater::startPreviewJob(const KFileItemList& items)
             break;
         }
     }
-    KJob* job = KIO::filePreview(itemSubSet, cacheSize, &m_enabledPlugins);
+    KIO::PreviewJob* job = new KIO::PreviewJob(itemSubSet, cacheSize, &m_enabledPlugins);
+    job->setIgnoreMaximumSize(items.first().isLocalFile());
+    if (job->ui()) {
+        job->ui()->setWindow(qApp->activeWindow());
+    }
 
     connect(job,  SIGNAL(gotPreview(KFileItem,QPixmap)),
             this, SLOT(slotGotPreview(KFileItem,QPixmap)));
@@ -553,10 +686,17 @@ void KFileItemModelRolesUpdater::resolvePendingRoles()
 {
     int resolvedCount = 0;
 
-    const bool hasSlowRoles = m_previewShown
-                              || m_roles.contains("size")
-                              || m_roles.contains("type")
-                              || m_roles.contains("isExpandable");
+    bool hasSlowRoles = m_previewShown;
+    if (!hasSlowRoles) {
+        QSetIterator<QByteArray> it(m_roles);
+        while (it.hasNext()) {
+            if (m_resolvableRoles.contains(it.next())) {
+                hasSlowRoles = true;
+                break;
+            }
+        }
+    }
+
     const ResolveHint resolveHint = hasSlowRoles ? ResolveFast : ResolveAll;
 
     // Resolving the MIME type can be expensive. Assure that not more than MaxBlockTimeout ms are
@@ -569,12 +709,12 @@ void KFileItemModelRolesUpdater::resolvePendingRoles()
     QSetIterator<KFileItem> visibleIt(m_pendingVisibleItems);
     while (visibleIt.hasNext()) {
         const KFileItem item = visibleIt.next();
-        applyResolvedRoles(item, resolveHint);
         if (!hasSlowRoles) {
             Q_ASSERT(!m_pendingInvisibleItems.contains(item));
-            // All roles have been resolved already by applyResolvedRoles()
+            // All roles will be resolved by applyResolvedRoles()
             m_pendingVisibleItems.remove(item);
         }
+        applyResolvedRoles(item, resolveHint);
         ++resolvedCount;
 
         if (timer.elapsed() > MaxBlockTimeout) {
@@ -625,6 +765,8 @@ void KFileItemModelRolesUpdater::resolvePendingRoles()
     }
     kDebug() << "[TIME] Resolved pending roles:" << timer.elapsed();
 #endif
+
+    applySortProgressToModel();
 }
 
 void KFileItemModelRolesUpdater::resetPendingRoles()
@@ -717,6 +859,56 @@ void KFileItemModelRolesUpdater::sortAndResolvePendingRoles()
     resolvePendingRoles();
 }
 
+void KFileItemModelRolesUpdater::applySortProgressToModel()
+{
+    if (m_sortingProgress < 0) {
+        return;
+    }
+
+    // Inform the model about the progress of the resolved items,
+    // so that it can give an indication when the sorting has been finished.
+    const int resolvedCount = m_model->count()
+                              - m_pendingVisibleItems.count()
+                              - m_pendingInvisibleItems.count();
+    if (resolvedCount > 0) {
+        m_model->emitSortProgress(resolvedCount);
+        if (resolvedCount == m_model->count()) {
+            m_sortingProgress = -1;
+        }
+    }
+}
+
+void KFileItemModelRolesUpdater::updateSortProgress()
+{
+    const QByteArray sortRole = m_model->sortRole();
+
+    // Optimization if the sorting is done by type: In case if all MIME-types
+    // are known, the types have been resolved already by KFileItemModel and
+    // no sort-progress feedback is required.
+    const bool showProgress = (sortRole == "type")
+                              ? hasUnknownMimeTypes()
+                              : m_resolvableRoles.contains(sortRole);
+
+    if (m_sortingProgress >= 0) {
+        // Mark the current sorting as finished
+        m_model->emitSortProgress(m_model->count());
+    }
+    m_sortingProgress = showProgress ? 0 : -1;
+}
+
+bool KFileItemModelRolesUpdater::hasUnknownMimeTypes() const
+{
+    const int count = m_model->count();
+    for (int i = 0; i < count; ++i) {
+        const KFileItem item = m_model->fileItem(i);
+        if (!item.isMimeTypeKnown()) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 bool KFileItemModelRolesUpdater::applyResolvedRoles(const KFileItem& item, ResolveHint hint)
 {
     if (item.isNull()) {
@@ -787,6 +979,35 @@ QHash<QByteArray, QVariant> KFileItemModelRolesUpdater::rolesData(const KFileIte
 
     data.insert("iconOverlays", item.overlays());
 
+#ifdef HAVE_NEPOMUK
+    if (m_nepomukResourceWatcher) {
+        const KNepomukRolesProvider& rolesProvider = KNepomukRolesProvider::instance();
+        Nepomuk::Resource resource(item.nepomukUri());
+        QHashIterator<QByteArray, QVariant> it(rolesProvider.roleValues(resource, m_roles));
+        while (it.hasNext()) {
+            it.next();
+            data.insert(it.key(), it.value());
+        }
+
+        QUrl uri = resource.resourceUri();
+        if (uri.isEmpty()) {
+            // TODO: Is there another way to explicitly create a resource?
+            // We need a resource to be able to track it for changes.
+            resource.setRating(0);
+            uri = resource.resourceUri();
+        }
+        if (!uri.isEmpty() && !m_nepomukUriItems.contains(uri)) {
+            m_nepomukResourceWatcher->addResource(resource);
+
+            if (m_nepomukUriItems.isEmpty()) {
+                m_nepomukResourceWatcher->start();
+            }
+
+            m_nepomukUriItems.insert(uri, item.url());
+        }
+    }
+#endif
+
     return data;
 }
 
@@ -829,7 +1050,7 @@ KFileItemList KFileItemModelRolesUpdater::sortedItems(const QSet<KFileItem>& ite
 int KFileItemModelRolesUpdater::subItemsCount(const QString& path) const
 {
     const bool countHiddenFiles = m_model->showHiddenFiles();
-    const bool showFoldersOnly  = m_model->showFoldersOnly();
+    const bool showFoldersOnly  = m_model->showDirectoriesOnly();
 
 #ifdef Q_WS_WIN
     QDir dir(path);
