@@ -23,10 +23,14 @@
 
 #include "kitemlistcontroller.h"
 
+#include <KGlobalSettings>
+#include <KDebug>
+
 #include "kitemlistview.h"
-#include "kitemlistrubberband_p.h"
 #include "kitemlistselectionmanager.h"
-#include "kitemlistkeyboardsearchmanager_p.h"
+
+#include "private/kitemlistrubberband.h"
+#include "private/kitemlistkeyboardsearchmanager.h"
 
 #include <QApplication>
 #include <QDrag>
@@ -37,10 +41,7 @@
 #include <QMimeData>
 #include <QTimer>
 
-#include <KGlobalSettings>
-#include <KDebug>
-
-KItemListController::KItemListController(QObject* parent) :
+KItemListController::KItemListController(KItemModelBase* model, KItemListView* view, QObject* parent) :
     QObject(parent),
     m_singleClickActivation(KGlobalSettings::singleClick()),
     m_selectionTogglePressed(false),
@@ -66,10 +67,18 @@ KItemListController::KItemListController(QObject* parent) :
     m_autoActivationTimer->setSingleShot(true);
     m_autoActivationTimer->setInterval(-1);
     connect(m_autoActivationTimer, SIGNAL(timeout()), this, SLOT(slotAutoActivationTimeout()));
+
+    setModel(model);
+    setView(view);
 }
 
 KItemListController::~KItemListController()
 {
+    setView(0);
+    Q_ASSERT(!m_view);
+
+    setModel(0);
+    Q_ASSERT(!m_model);
 }
 
 void KItemListController::setModel(KItemModelBase* model)
@@ -79,7 +88,14 @@ void KItemListController::setModel(KItemModelBase* model)
     }
 
     KItemModelBase* oldModel = m_model;
+    if (oldModel) {
+        oldModel->deleteLater();
+    }
+
     m_model = model;
+    if (m_model) {
+        m_model->setParent(this);
+    }
 
     if (m_view) {
         m_view->setModel(m_model);
@@ -109,14 +125,17 @@ void KItemListController::setView(KItemListView* view)
     KItemListView* oldView = m_view;
     if (oldView) {
         disconnect(oldView, SIGNAL(scrollOffsetChanged(qreal,qreal)), this, SLOT(slotViewScrollOffsetChanged(qreal,qreal)));
+        oldView->deleteLater();
     }
 
     m_view = view;
 
     if (m_view) {
+        m_view->setParent(this);
         m_view->setController(this);
         m_view->setModel(m_model);
         connect(m_view, SIGNAL(scrollOffsetChanged(qreal,qreal)), this, SLOT(slotViewScrollOffsetChanged(qreal,qreal)));
+        updateExtendedSelectionRegion();
     }
 
     emit viewChanged(m_view, oldView);
@@ -130,6 +149,7 @@ KItemListView* KItemListController::view() const
 void KItemListController::setSelectionBehavior(SelectionBehavior behavior)
 {
     m_selectionBehavior = behavior;
+    updateExtendedSelectionRegion();
 }
 
 KItemListController::SelectionBehavior KItemListController::selectionBehavior() const
@@ -471,9 +491,7 @@ bool KItemListController::mousePressEvent(QGraphicsSceneMouseEvent* event, const
 
     m_pressedMousePos = transform.map(event->pos());
     m_pressedIndex = m_view->itemAt(m_pressedMousePos);
-    if (m_pressedIndex >= 0) {
-        emit itemPressed(m_pressedIndex, event->button());
-    }
+    emit mouseButtonPressed(m_pressedIndex, event->buttons());
 
     if (m_view->isAboveExpansionToggle(m_pressedIndex, m_pressedMousePos)) {
         m_selectionManager->endAnchoredSelection();
@@ -654,9 +672,7 @@ bool KItemListController::mouseReleaseEvent(QGraphicsSceneMouseEvent* event, con
         return false;
     }
 
-    if (m_pressedIndex >= 0) {
-        emit itemReleased(m_pressedIndex, event->button());
-    }
+    emit mouseButtonReleased(m_pressedIndex, event->buttons());
 
     const bool isAboveSelectionToggle = m_view->isAboveSelectionToggle(m_pressedIndex, m_pressedMousePos);
     if (isAboveSelectionToggle) {
@@ -750,6 +766,7 @@ bool KItemListController::dragLeaveEvent(QGraphicsSceneDragDropEvent* event, con
     Q_UNUSED(transform);
 
     m_view->setAutoScroll(false);
+    m_view->hideDropIndicator();
 
     KItemListWidget* widget = hoveredWidget();
     if (widget) {
@@ -785,6 +802,11 @@ bool KItemListController::dragMoveEvent(QGraphicsSceneDragDropEvent* event, cons
             const int index = newHoveredWidget->index();
             if (m_model->supportsDropping(index)) {
                 newHoveredWidget->setHovered(true);
+            } else if (m_model->sortRole().isEmpty()) {
+                // The model supports inserting of items on
+                // the given index. Assure that a drop-indicator
+                // is shown by the view.
+                m_view->showDropIndicator(pos);
             }
             emit itemHovered(index);
 
@@ -809,8 +831,15 @@ bool KItemListController::dropEvent(QGraphicsSceneDragDropEvent* event, const QT
     m_view->setAutoScroll(false);
 
     const QPointF pos = transform.map(event->pos());
-    const int index = m_view->itemAt(pos);
-    emit itemDropEvent(index, event);
+    if (m_model->sortRole().isEmpty()) {
+        // The model supports inserting of items on
+        // a given index.
+        const int dropIndex = m_view->showDropIndicator(pos);
+        m_view->hideDropIndicator();
+        emit itemDropEvent(dropIndex, event);
+    } else {
+        emit itemDropEvent(m_view->itemAt(pos), event);
+    }
 
     return true;
 }
@@ -1055,6 +1084,11 @@ void KItemListController::startDragging()
     const QPixmap pixmap = m_view->createDragPixmap(selectedItems);
     drag->setPixmap(pixmap);
 
+    // TODO: The vertical hotspot of -24 should be replaced by the
+    // height of the QCursor-pixmap.
+    const QPoint hotSpot(pixmap.width() / 2, -24);
+    drag->setHotSpot(hotSpot);
+
     drag->exec(Qt::MoveAction | Qt::CopyAction | Qt::LinkAction, Qt::CopyAction);
 }
 
@@ -1179,4 +1213,16 @@ qreal KItemListController::keyboardAnchorPos(int index) const
     return 0;
 }
 
+void KItemListController::updateExtendedSelectionRegion()
+{
+    if (m_view) {
+        const bool extend = (m_selectionBehavior != MultiSelection);
+        KItemListStyleOption option = m_view->styleOption();
+        if (option.extendedSelectionRegion != extend) {
+            option.extendedSelectionRegion = extend;
+            m_view->setStyleOption(option);
+        }
+    }
+}
+
 #include "kitemlistcontroller.moc"
